from __future__ import print_function

import logging

log = logging.getLogger('SKQ.Common')

__all__ = ['Result', 'Stats']

from ._objective import ObjectiveFunction
from ._result import Result
from ._stats import Stats



from __future__ import print_function

__all__ = ['ObjectiveFunction']

from ._stats import Stats
import numpy


class ObjectiveFunction(object):
    def __init__(self, func, options = {}):
        self.simple_function = False

        self.objective = func
        self.stats = Stats()
        try:
            for key, value in options.items():
                setattr(self, key, value)
        except:
            pass   # options is not a dict

    def __call__(self, par):
        self.stats.nevals += 1

        result = self.objective(par)
        try:
            nres = len(result)
        except TypeError:
            nres = 0

        err_estimate = 0.; cost = 1
        if nres == 0:
            val = result
        else:
            val = result[0]
            if nres >= 2:
                err_estimate = result[1]
            if nres >= 3:
                cost = result[2]
        self.stats.add_history(val, par)

        if self.simple_function:
            return val
        return result

    def get_history(self):
        return self.stats.full_history()



from __future__ import print_function

__all__ = ['Result']


class Result(object):
    def __init__(self, val, par):
        self.optval = val
        self.optpar = par

    def __str__(self):
        return "Optimal value: %g, at parameters: %s" % (self.optval, str(self.optpar))



import py, os, sys
import numpy as np

sys.path = [os.path.join(os.pardir, 'python')] + sys.path


class TestCOMMON:
    def setup_class(cls):
        import SQCommon, logging
        SQCommon.log.setLevel(logging.DEBUG)

    def test01_API(self):
        """Test existence and semantics of the common API"""

        import SQCommon

        assert SQCommon.Result
        assert SQCommon.Stats



from __future__ import print_function

import numpy

__all__ = ['Result']


class Stats(object):
    def __init__(self, nevals = 0):
        self.nevals = nevals
        self._history = list()

    def add_history(self, fval, par):
        self._history.append([fval]+list(numpy.atleast_1d(par)))

    def full_history(self):
        """Return the full call history. Each row represents one call, with
           the first column the result from the objective function call, the
           other columns the parameters used."""

        return numpy.array(self._history)
